@@ -57,6 +57,7 @@ static void indent(char ic)
 	indent_level++;
 }
 
+
 static void stringdump(const char *name, const char *member)
 /* dump a string member with current indent */
 {
@@ -229,7 +230,11 @@ void dump_config(struct runctl *runp, struct query *querylist)
 
 	    using_kpop =
 		(ctl->server.protocol == P_POP3 &&
+#if !INET6
 		 ctl->server.port == KPOP_PORT &&
+#else
+		 0 == strcmp( ctl->server.service, KPOP_PORT ) &&
+#endif
 		 ctl->server.preauthenticate == A_KERBEROS_V4);
 
 	    stringdump("pollname", ctl->server.pollname); 
@@ -237,7 +242,11 @@ void dump_config(struct runctl *runp, struct query *querylist)
 	    stringdump("via", ctl->server.via); 
 	    stringdump("protocol", 
 		       using_kpop ? "KPOP" : showproto(ctl->server.protocol));
+#if !INET6
 	    numdump("port",  ctl->server.port);
+#else
+	    stringdump("service", ctl->server.service); 
+#endif
 	    numdump("timeout",  ctl->server.timeout);
 	    numdump("interval", ctl->server.interval);
 
@@ -270,6 +279,9 @@ void dump_config(struct runctl *runp, struct query *querylist)
 	    stringdump("monitor", ctl->server.monitor);
 #endif /* linux */
 
+	    stringdump("plugin", ctl->server.plugin);
+	    stringdump("plugout", ctl->server.plugout);
+
 	    indent(0);
 	    fputs("'users': ", stdout);
 	    indent('[');
@@ -310,8 +322,15 @@ void dump_config(struct runctl *runp, struct query *querylist)
 	booldump("mimedecode", ctl->mimedecode);
 
 	stringdump("mda", ctl->mda);
+	stringdump("bsmtp", ctl->bsmtp);
+	indent('\0');
+	if (ctl->listener == LMTP_MODE)
+	    fputs("'lmtp':TRUE,", stdout);
+	else
+	    fputs("'lmtp':FALSE,", stdout);
+	    
 #ifdef INET6
-	stringdump("netsec", ctl->netsec);
+	stringdump("netsec", ctl->server.netsec);
 #endif /* INET6 */
 	stringdump("preconnect", ctl->preconnect);
 	stringdump("postconnect", ctl->postconnect);
