@@ -12,6 +12,7 @@
 #include  <netdb.h>
 #include  <errno.h>
 #include  <unistd.h>
+#include  "i18n.h"
 #include  "fetchmail.h"
 #include  "smtp.h"
 #include  "socket.h"
@@ -36,15 +37,15 @@ static int etrn_getrange(int sock, struct query *ctl, const char *id,
     char buf [MSGBUFSIZE+1];
     struct idlist *qnp;		/* pointer to Q names */
 
-    if ((ok = SMTP_ehlo(sock, ctl->server.truename, &opts)))
+    if ((ok = SMTP_ehlo(sock, fetchmailhost, &opts)))
     {
-	error(0, 0, "%s's SMTP listener does not support ESMTP",
+	report(stderr, 0, _("%s's SMTP listener does not support ESMTP"),
 	      ctl->server.pollname);
 	return(ok);
     }
     else if (!(opts & ESMTP_ETRN))
     {
-	error(0, 0, "%s's SMTP listener does not support ETRN",
+	report(stderr, 0, _("%s's SMTP listener does not support ETRN"),
 	      ctl->server.pollname);
 	return(PS_PROTOCOL);
     }
@@ -67,36 +68,39 @@ static int etrn_getrange(int sock, struct query *ctl, const char *id,
 	switch(atoi(buf))
 	{
 	case 250:	/* OK, queuing for node <x> started */
-	    error(0, 0, "Queuing for %s started", qnp->id);
+	    if (outlevel >= O_SILENT)
+		report(stdout, 0, _("Queuing for %s started"), qnp->id);
 	    break;
 
 	case 251:	/* OK, no messages waiting for node <x> */
-	    error(0, 0, "No messages waiting for %s", qnp->id);
+	    if (outlevel >= O_SILENT)
+		report(stdout, 0, _("No messages waiting for %s"), qnp->id);
 	    return(PS_NOMAIL);
 
 	case 252:	/* OK, pending messages for node <x> started */
 	case 253:	/* OK, <n> pending messages for node <x> started */
-	    error(0, 0, "Pending messages for %s started", qnp->id);
+	    if (outlevel >= O_SILENT)
+		report(stdout, 0, _("Pending messages for %s started"), qnp->id);
 	    break;
 
 	case 458:	/* Unable to queue messages for node <x> */
-	    error(0, -1, "Unable to queue messages for node %s",qnp->id);
+	    report(stderr, 0, _("Unable to queue messages for node %s"),qnp->id);
 	    return(PS_PROTOCOL);
 
 	case 459:	/* Node <x> not allowed: <reason> */
-	    error(0, -1, "Node %s not allowed: %s", qnp->id, buf);
+	    report(stderr, 0, _("Node %s not allowed: %s"), qnp->id, buf);
 	    return(PS_AUTHFAIL);
 
 	case 500:	/* Syntax Error */
-	    error(0, -1, "ETRN syntax error");
+	    report(stderr, 0, _("ETRN syntax error"));
 	    return(PS_PROTOCOL);
 
 	case 501:	/* Syntax Error in Parameters */
-	    error(0, -1, "ETRN syntax error in parameters");
+	    report(stderr, 0, _("ETRN syntax error in parameters"));
 	    return(PS_PROTOCOL);
 
 	default:
-	    error(0, -1, "Unknown ETRN error %d", atoi(buf));
+	    report(stderr, 0, _("Unknown ETRN error %d"), atoi(buf));
 	    return(PS_PROTOCOL);
 	}
     }
@@ -140,19 +144,19 @@ int doETRN (struct query *ctl)
     int status;
 
     if (ctl->keep) {
-	fprintf(stderr, "Option --keep is not supported with ETRN\n");
+	fprintf(stderr, _("Option --keep is not supported with ETRN\n"));
 	return(PS_SYNTAX);
     }
     if (ctl->flush) {
-	fprintf(stderr, "Option --flush is not supported with ETRN\n");
+	fprintf(stderr, _("Option --flush is not supported with ETRN\n"));
 	return(PS_SYNTAX);
     }
     if (ctl->mailboxes->id) {
-	fprintf(stderr, "Option --remote is not supported with ETRN\n");
+	fprintf(stderr, _("Option --remote is not supported with ETRN\n"));
 	return(PS_SYNTAX);
     }
     if (check_only) {
-	fprintf(stderr, "Option --check is not supported with ETRN\n");
+	fprintf(stderr, _("Option --check is not supported with ETRN\n"));
 	return(PS_SYNTAX);
     }
     peek_capable = FALSE;
