@@ -34,11 +34,16 @@
 #include <netdb.h>
 #endif /* HAVE_GETHOSTBYNAME */
 
+#ifdef HESIOD
+#include <hesiod.h>
+#endif
+
 #include "fetchmail.h"
 #include "tunable.h"
 #include "smtp.h"
 #include "getopt.h"
 #include "netrc.h"
+#include "i18n.h"
 
 #ifndef ENETUNREACH
 #define ENETUNREACH   128       /* Interactive doesn't know this */
@@ -124,7 +129,7 @@ static void unlockit(void)
 int itimerflag;
 void itimerthread(void* dummy) {
   if (outlevel >= O_VERBOSE)
-    fprintf(stderr, "fetchmail: thread sleeping for %d sec.\n", poll_interval);
+    fprintf(stderr, _("fetchmail: thread sleeping for %d sec.\n"), poll_interval);
   while(1) {
     _sleep2(poll_interval*1000);
     kill((getpid()), SIGALRM);
@@ -143,6 +148,10 @@ int main (int argc, char **argv)
     pid_t pid;
 
     envquery(argc, argv);
+#ifdef ENABLE_NLS
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
+#endif
 
 #define IDFILE_NAME	".fetchids"
     run.idfile = (char *) xmalloc(strlen(home)+strlen(IDFILE_NAME)+2);
@@ -155,12 +164,9 @@ int main (int argc, char **argv)
     if ((parsestatus = parsecmdline(argc,argv, &cmd_run, &cmd_opts)) < 0)
 	exit(PS_SYNTAX);
 
-    /* this hint to stdio should help messages come out in the right order */
-    setvbuf(stdout, NULL, _IOLBF, MSGBUFSIZE);
-
     if (versioninfo)
     {
-	printf("This is fetchmail release %s", RELEASE_ID);
+	printf(_("This is fetchmail release %s"), VERSION);
 #ifdef POP2_ENABLE
 	printf("+POP2");
 #endif /* POP2_ENABLE */
@@ -191,6 +197,14 @@ int main (int argc, char **argv)
 #if NET_SECURITY
 	printf("+NETSEC");
 #endif /* NET_SECURITY */
+#ifdef HAVE_SOCKS
+  #if HAVE_SOCKS
+	printf("+SOCKS");
+  #endif
+#endif /* HAVE_SOCKS */
+#if ENABLE_NLS
+	printf("+NLS");
+#endif /* ENABLE_NLS */
 	putchar('\n');
 
 	/* this is an attempt to help remote debugging */
@@ -203,10 +217,12 @@ int main (int argc, char **argv)
 
     /* set up to do lock protocol */
 #define	FETCHMAIL_PIDFILE	"fetchmail.pid"
-    xalloca(tmpbuf, char *, strlen(home) + strlen(FETCHMAIL_PIDFILE) + 3);
-    if (!getuid())
+    if (!getuid()) {
+	xalloca(tmpbuf, char *,
+		strlen(PID_DIR) + strlen(FETCHMAIL_PIDFILE) + 2);
 	sprintf(tmpbuf, "%s/%s", PID_DIR, FETCHMAIL_PIDFILE);
-    else {
+    } else {
+	xalloca(tmpbuf, char *, strlen(home) + strlen(FETCHMAIL_PIDFILE) + 3);
 	strcpy(tmpbuf, home);
 	strcat(tmpbuf, "/.");
 	strcat(tmpbuf, FETCHMAIL_PIDFILE);
@@ -216,17 +232,17 @@ int main (int argc, char **argv)
     /* perhaps we just want to check options? */
     if (versioninfo)
     {
-	printf("Taking options from command line");
+	printf(_("Taking options from command line"));
 	if (access(rcfile, 0))
 	    printf("\n");
 	else
-	    printf(" and %s\n", rcfile);
+	    printf(_(" and %s\n"), rcfile);
 	if (outlevel >= O_VERBOSE)
-	    printf("Lockfile at %s\n", tmpbuf);
+	    printf(_("Lockfile at %s\n"), tmpbuf);
 
 	if (querylist == NULL)
 	    (void) fprintf(stderr,
-		"No mailservers set up -- perhaps %s is missing?\n", rcfile);
+		_("No mailservers set up -- perhaps %s is missing?\n"), rcfile);
 	else
 	    dump_params(&run, querylist, implicitmode);
 	exit(0);
@@ -243,7 +259,7 @@ int main (int argc, char **argv)
     pid = -1;
     if ((lockfile = (char *) malloc(strlen(tmpbuf) + 1)) == NULL)
     {
-	fprintf(stderr,"fetchmail: cannot allocate memory for lock name.\n");
+	fprintf(stderr,_("fetchmail: cannot allocate memory for lock name.\n"));
 	exit(PS_EXCLUDE);
     }
     else
@@ -253,7 +269,7 @@ int main (int argc, char **argv)
 	bkgd = (fscanf(lockfp,"%d %d", &pid, &st) == 2);
 
 	if (kill(pid, 0) == -1) {
-	    fprintf(stderr,"fetchmail: removing stale lockfile\n");
+	    fprintf(stderr,_("fetchmail: removing stale lockfile\n"));
 	    pid = -1;
 	    bkgd = FALSE;
 	    unlink(lockfile);
@@ -263,7 +279,7 @@ int main (int argc, char **argv)
 
     /* if no mail servers listed and nothing in background, we're done */
     if (!(quitmode && argc == 2) && pid == -1 && querylist == NULL) {
-	(void)fputs("fetchmail: no mailservers have been specified.\n",stderr);
+	(void)fputs(_("fetchmail: no mailservers have been specified.\n"),stderr);
 	exit(PS_SYNTAX);
     }
 
@@ -272,20 +288,20 @@ int main (int argc, char **argv)
     {
 	if (pid == -1) 
 	{
-	    fprintf(stderr,"fetchmail: no other fetchmail is running\n");
+	    fprintf(stderr,_("fetchmail: no other fetchmail is running\n"));
 	    if (argc == 2)
 		exit(PS_EXCLUDE);
 	}
 	else if (kill(pid, SIGTERM) < 0)
 	{
-	    fprintf(stderr,"fetchmail: error killing %s fetchmail at %d; bailing out.\n",
-		    bkgd ? "background" : "foreground", pid);
+	    fprintf(stderr,_("fetchmail: error killing %s fetchmail at %d; bailing out.\n"),
+		    bkgd ? _("background") : _("foreground"), pid);
 	    exit(PS_EXCLUDE);
 	}
 	else
 	{
-	    fprintf(stderr,"fetchmail: %s fetchmail at %d killed.\n",
-		    bkgd ? "background" : "foreground", pid);
+	    fprintf(stderr,_("fetchmail: %s fetchmail at %d killed.\n"),
+		    bkgd ? _("background") : _("foreground"), pid);
 	    unlink(lockfile);
 	    if (argc == 2)
 		exit(0);
@@ -300,33 +316,33 @@ int main (int argc, char **argv)
 	if (check_only)
 	{
 	    fprintf(stderr,
-		 "fetchmail: can't check mail while another fetchmail to same host is running.\n");
+		 _("fetchmail: can't check mail while another fetchmail to same host is running.\n"));
 	    return(PS_EXCLUDE);
         }
 	else if (!implicitmode)
 	{
 	    fprintf(stderr,
-		 "fetchmail: can't poll specified hosts with another fetchmail running at %d.\n",
+		 _("fetchmail: can't poll specified hosts with another fetchmail running at %d.\n"),
 		 pid);
 		return(PS_EXCLUDE);
 	}
 	else if (!bkgd)
 	{
 	    fprintf(stderr,
-		 "fetchmail: another foreground fetchmail is running at %d.\n",
+		 _("fetchmail: another foreground fetchmail is running at %d.\n"),
 		 pid);
 		return(PS_EXCLUDE);
 	}
 	else if (argc > 1)
 	{
 	    fprintf(stderr,
-		    "fetchmail: can't accept options while a background fetchmail is running.\n");
+		    _("fetchmail: can't accept options while a background fetchmail is running.\n"));
 	    return(PS_EXCLUDE);
 	}
 	else if (kill(pid, SIGUSR1) == 0)
 	{
 	    fprintf(stderr,
-		    "fetchmail: background fetchmail at %d awakened.\n",
+		    _("fetchmail: background fetchmail at %d awakened.\n"),
 		    pid);
 	    return(0);
 	}
@@ -338,7 +354,7 @@ int main (int argc, char **argv)
 	     * SIGUSR1/SIGHUP transmission.
 	     */
 	    fprintf(stderr,
-		    "fetchmail: elder sibling at %d died mysteriously.\n",
+		    _("fetchmail: elder sibling at %d died mysteriously.\n"),
 		    pid);
 	    return(PS_UNDEFINED);
 	}
@@ -367,6 +383,7 @@ int main (int argc, char **argv)
 
     /* pick up interactively any passwords we need but don't have */ 
     for (ctl = querylist; ctl; ctl = ctl->next)
+    {
 	if (ctl->active && !(implicitmode && ctl->server.skip)&&!ctl->password)
 	{
 	    if (ctl->server.preauthenticate == A_KERBEROS_V4 ||
@@ -380,14 +397,25 @@ int main (int argc, char **argv)
 		ctl->password = ctl->remotename;
 	    else
 	    {
-		/* look up the host and account in the .netrc file. */
-		netrc_entry *p = search_netrc(netrc_list,ctl->server.pollname);
+		netrc_entry *p;
+
+		/* look up the pollname and account in the .netrc file. */
+		p = search_netrc(netrc_list, ctl->server.pollname);
 		while (p && strcmp(p->account, ctl->remotename))
 		    p = search_netrc(p->next, ctl->remotename);
-
 		/* if we find a matching entry with a password, use it */
 		if (p && p->password)
 		    ctl->password = xstrdup(p->password);
+
+		/* otherwise try with "via" name if there is one */
+		else if (ctl->server.via)
+		{
+		    p = search_netrc(netrc_list, ctl->server.via);
+		    while (p && strcmp(p->account, ctl->remotename))
+		        p = search_netrc(p->next, ctl->remotename);
+		    if (p && p->password)
+		        ctl->password = xstrdup(p->password);
+		}
 	    }
 
 	    if (ctl->server.protocol != P_ETRN && ctl->server.protocol != P_IMAP_K4
@@ -396,16 +424,28 @@ int main (int argc, char **argv)
 #endif /* GSSAPI */
                 && !ctl->password)
 	    {
-#define	PASSWORD_PROMPT	"Enter password for %s@%s: "
-		xalloca(tmpbuf, char *, strlen(PASSWORD_PROMPT) +
+		char* password_prompt = _("Enter password for %s@%s: ");
+
+		xalloca(tmpbuf, char *, strlen(password_prompt) +
 					strlen(ctl->remotename) +
 					strlen(ctl->server.pollname) + 1);
-		(void) sprintf(tmpbuf, PASSWORD_PROMPT,
+		(void) sprintf(tmpbuf, password_prompt,
 			       ctl->remotename, ctl->server.pollname);
 		ctl->password = xstrdup((char *)getpassword(tmpbuf));
-#undef	PASSWORD_PROMPT
 	    }
 	}
+    }
+
+/* Time to initiate the SOCKS library (this is not mandatory: it just
+ registers the correct application name for logging purpose. If you
+ have some problem, comment these lines). */
+#ifdef HAVE_SOCKS
+  #if HAVE_SOCKS
+/* Mmmh... I don't like hardcoded application names,
+ but "fetchmail" is everywhere... */
+    SOCKSinit("fetchmail");
+  #endif
+#endif /* HAVE_SOCKS */
 
     /*
      * Maybe time to go to demon mode...
@@ -414,17 +454,17 @@ int main (int argc, char **argv)
     if (run.use_syslog)
     {
     	openlog(program_name, LOG_PID, LOG_MAIL);
-	error_init(-1);
+	report_init(-1);
     }
     else
 #endif
-	error_init((run.poll_interval == 0 || nodetach) && !run.logfile);
+	report_init((run.poll_interval == 0 || nodetach) && !run.logfile);
 
     if (run.poll_interval)
     {
 	if (!nodetach)
 	    daemonize(run.logfile, termhook);
-	error( 0, 0, "starting fetchmail %s daemon ", RELEASE_ID);
+	report(stdout, 0, _("starting fetchmail %s daemon "), VERSION);
 
 	/*
 	 * We'll set up a handler for these when we're sleeping,
@@ -483,14 +523,22 @@ int main (int argc, char **argv)
 	{
 	    if (ctl->active && !(implicitmode && ctl->server.skip))
 	    {
+		if (ctl->wedged)
+		{
+		    report(stderr, 0, 
+			  _("poll of %s skipped (failed authentication or too many timeouts)"),
+			  ctl->server.pollname);
+		    continue;
+		}
+
 		/* check skip interval first so that it counts all polls */
 		if (run.poll_interval && ctl->server.interval) 
 		{
 		    if (ctl->server.poll_count++ % ctl->server.interval) 
 		    {
 			if (outlevel >= O_VERBOSE)
-			    fprintf(stderr,
-				    "fetchmail: interval not reached, not querying %s\n",
+			    report(stdout, 0,
+				    _("interval not reached, not querying %s"),
 				    ctl->server.pollname);
 			continue;
 		    }
@@ -517,18 +565,13 @@ int main (int argc, char **argv)
 		    {
 			write_saved_lists(querylist, run.idfile);
 			if (outlevel >= O_DEBUG)
-			    error(0, 0, "saved UID List");
+			    report(stdout, 0, _("saved UID List"));
 		    }
 #endif  /* POP3_ENABLE */
 		}
-		else if (querystatus == PS_AUTHFAIL) {
-		    /* have it logged with priority LOG_ALERT */
-		    error(querystatus, -1, "Authentication failed for %s@%s",
-				ctl->remotename, visbuf(ctl->server.pollname));
-		}
 		else if (!check_only && 
 			 ((querystatus!=PS_NOMAIL) || (outlevel==O_DEBUG)))
-		    error(0, 0, "Query status=%d", querystatus);
+		    report(stdout, 0, _("Query status=%d"), querystatus);
 
 #if defined(linux) && !INET6
 		if (ctl->server.monitor)
@@ -570,14 +613,28 @@ int main (int argc, char **argv)
 	 */
 	if (run.poll_interval)
 	{
-	    if (outlevel >= O_VERBOSE)
-	    {
-		time_t	now;
+	    /* 
+	     * Because passwords can expire, it may happen that *all*
+	     * hosts are now out of the loop due to authfail
+	     * conditions.  If this happens daemon-mode fetchmail
+	     * should softly and silently vanish away, rather than
+	     * spinning uselessly.
+	     */
+	    int unwedged = 0;
 
-		time(&now);
-		fprintf(stderr, "fetchmail: sleeping at %s", ctime(&now));
+	    for (ctl = querylist; ctl; ctl = ctl->next)
+		if (ctl->active && !(implicitmode && ctl->server.skip))
+		    if (!ctl->wedged)
+			unwedged++;
+	    if (!unwedged)
+	    {
+		report(stderr, 0, _("All connections are wedged.  Exiting."));
+		exit(PS_AUTHFAIL);
 	    }
 
+	    if (outlevel >= O_VERBOSE)
+		report(stdout, 0, _("fetchmail: sleeping at %s"), rfc822timestamp());
+
 	    /*
 	     * With this simple hack, we make it possible for a foreground 
 	     * fetchmail to wake up one in daemon mode.  What we want is the
@@ -589,15 +646,16 @@ int main (int argc, char **argv)
 	    if (!getuid())
 		signal(SIGHUP, donothing);
 
-	    /*
-	     * We can't use sleep(3) here because we need an alarm(3)
-	     * equivalent in order to implement server nonresponse timeout.
-	     * We'll just assume setitimer(2) is available since fetchmail
-	     * has to have a BSDoid socket layer to work at all.
-	     */
+	    /* time for a pause in the action... */
 	    {
 #ifndef __EMX__
 #ifdef SLEEP_WITH_ALARM		/* not normally on */
+		/*
+		 * We can't use sleep(3) here because we need an alarm(3)
+		 * equivalent in order to implement server nonresponse timeout.
+		 * We'll just assume setitimer(2) is available since fetchmail
+		 * has to have a BSDoid socket layer to work at all.
+		 */
 		/* 
 		 * This code stopped working under glibc-2, apparently due
 		 * to the change in signal(2) semantics.  (The siginterrupt
@@ -670,10 +728,14 @@ int main (int argc, char **argv)
 			|| ((run.poll_interval && !getuid()) && lastsig == SIGHUP))
 		{
 #ifdef SYS_SIGLIST_DECLARED
-		    error(0, 0, "awakened by %s", sys_siglist[lastsig]);
+		    report(stdout, 0, _("awakened by %s"), sys_siglist[lastsig]);
 #else
-		    error(0, 0, "awakened by signal %d", lastsig);
+		    report(stdout, 0, _("awakened by signal %d"), lastsig);
 #endif
+		    /* received a wakeup - unwedge all servers in case */
+		    /* the problem has been manually repaired          */
+		    for (ctl = querylist; ctl; ctl = ctl->next)
+		        ctl->wedged = FALSE;
 		}
 	    }
 
@@ -683,18 +745,13 @@ int main (int argc, char **argv)
 		signal(SIGHUP, SIG_IGN);
 
 	    if (outlevel >= O_VERBOSE)
-	    {
-		time_t	now;
-
-		time(&now);
-		fprintf(stderr, "fetchmail: awakened at %s", ctime(&now));
-	    }
+		report(stdout, 0, _("awakened at %s"), rfc822timestamp());
 	}
     } while
 	(run.poll_interval);
 
     if (outlevel >= O_VERBOSE)
-	fprintf(stderr,"fetchmail: normal termination, status %d\n",
+	report(stdout, 0, _("normal termination, status %d"),
 		successes ? PS_SUCCESS : querystatus);
 
     termhook(0);
@@ -745,10 +802,14 @@ static void optmerge(struct query *h2, struct query *h1, int force)
     FLAG_MERGE(server.interface_pair);
 #endif /* linux */
 
+    FLAG_MERGE(server.plugin);
+    FLAG_MERGE(server.plugout);
     FLAG_MERGE(wildcard);
     FLAG_MERGE(remotename);
     FLAG_MERGE(password);
     FLAG_MERGE(mda);
+    FLAG_MERGE(bsmtp);
+    FLAG_MERGE(listener);
     FLAG_MERGE(smtpaddress);
     FLAG_MERGE(preconnect);
     FLAG_MERGE(postconnect);
@@ -790,6 +851,7 @@ static int load_params(int argc, char **argv, int optind)
     def_opts.warnings = WARNING_INTERVAL;
     def_opts.remotename = user;
     def_opts.expunge = 1;
+    def_opts.listener = SMTP_MODE;
 
     /* this builds the host list */
     if (prc_parse_file(rcfile, !versioninfo) != 0)
@@ -816,7 +878,7 @@ static int load_params(int argc, char **argv, int optind)
 		{
 		    /* Is this correct? */
 		    if(predeclared)
-			fprintf(stderr,"Warning: multiple mentions of host %s in config file\n",argv[optind]);
+			fprintf(stderr,_("Warning: multiple mentions of host %s in config file\n"),argv[optind]);
 		    ctl->active = TRUE;
 		    predeclared = TRUE;
 		}
@@ -857,6 +919,8 @@ static int load_params(int argc, char **argv, int optind)
     /* merge in wired defaults, do sanity checks and prepare internal fields */
     for (ctl = querylist; ctl; ctl = ctl->next)
     {
+	ctl->wedged = FALSE;
+
 	if (configdump || (ctl->active && !(implicitmode && ctl->server.skip)))
 	{
 	    /* merge in defaults */
@@ -922,7 +986,7 @@ static int load_params(int argc, char **argv, int optind)
 	    if (ctl->localnames && ctl->localnames->next && ctl->server.dns)
 	    {
 		ctl->server.dns = FALSE;
-		fprintf(stderr, "fetchmail: warning: no DNS available to check multidrop fetches from %s\n", ctl->server.pollname);
+		fprintf(stderr, _("fetchmail: warning: no DNS available to check multidrop fetches from %s\n"), ctl->server.pollname);
 	    }
 #endif /* !HAVE_GETHOSTBYNAME || !HAVE_RES_SEARCH */
 
@@ -947,6 +1011,26 @@ static int load_params(int argc, char **argv, int optind)
 	    else
 		ctl->server.queryname = xstrdup(ctl->server.pollname);
 
+#ifdef HESIOD
+        /* If either the pollname or vianame are "hesiod" we want to
+           lookup the user's hesiod pobox host */
+
+        if (!strcasecmp(ctl->server.queryname, "hesiod")) {
+            struct hes_postoffice *hes_p;
+            hes_p = hes_getmailhost(ctl->remotename);
+            if (hes_p != NULL && strcmp(hes_p->po_type, "POP") == 0) {
+                 free(ctl->server.queryname);
+                 ctl->server.queryname = xstrdup(hes_p->po_host);
+                 if (ctl->server.via)
+                     free(ctl->server.via);
+                 ctl->server.via = xstrdup(hes_p->po_host);
+            } else {
+                 report(stderr, errno, _("couldn't find HESIOD pobox for %s"),
+                     ctl->remotename);
+            }
+        }
+#endif /* HESIOD */
+
 	    /*
 	     * We may have to canonicalize the server truename for later use.
 	     * Do this just once for each lead server, if necessary, in order
@@ -966,8 +1050,8 @@ static int load_params(int argc, char **argv, int optind)
 		namerec = gethostbyname(ctl->server.queryname);
 		if (namerec == (struct hostent *)NULL)
 		{
-		    error(0, errno,
-			  "couldn't find canonical DNS name of %s",
+		    report(stderr, errno,
+			  _("couldn't find canonical DNS name of %s"),
 			  ctl->server.pollname);
 		    exit(PS_DNS);
 		}
@@ -991,17 +1075,39 @@ static int load_params(int argc, char **argv, int optind)
 	    if (ctl->server.port < 0)
 	    {
 		(void) fprintf(stderr,
-			       "%s configuration invalid, port number cannot be negative",
+			       _("%s configuration invalid, port number cannot be negative\n"),
 			       ctl->server.pollname);
 		exit(PS_SYNTAX);
 	    }
 	    if (ctl->server.protocol == P_RPOP && ctl->server.port >= 1024)
 	    {
 		(void) fprintf(stderr,
-			       "%s configuration invalid, RPOP requires a privileged port",
+			       _("%s configuration invalid, RPOP requires a privileged port\n"),
 			       ctl->server.pollname);
 		exit(PS_SYNTAX);
 	    }
+	    if (ctl->listener == LMTP_MODE)
+	    {
+		struct idlist	*idp;
+
+		for (idp = ctl->smtphunt; idp; idp = idp->next)
+		{
+		    char	*cp;
+
+		    if (!(cp = strrchr(idp->id, '/')) ||
+#ifdef INET6 
+				(strcmp(++cp, SMTP_PORT) == 0))
+#else
+				(atoi(++cp) == SMTP_PORT))
+#endif /* INET6 */
+		    {
+			(void) fprintf(stderr,
+				       _("%s configuration invalid, LMTP can't use default SMTP port\n"),
+				       ctl->server.pollname);
+			exit(PS_SYNTAX);
+		    }
+		}
+	    }
 #endif /* !INET6 */
 	}
     }
@@ -1017,6 +1123,8 @@ static int load_params(int argc, char **argv, int optind)
 	run.invisible = cmd_run.invisible;
     if (cmd_run.use_syslog)
 	run.use_syslog = (cmd_run.use_syslog == FLAG_TRUE);
+    if (cmd_run.postmaster)
+	run.postmaster = cmd_run.postmaster;
 
     /* check and daemon options are not compatible */
     if (check_only && run.poll_interval)
@@ -1043,7 +1151,7 @@ static int load_params(int argc, char **argv, int optind)
     return(implicitmode);
 }
 
-void termhook(int sig)
+static void termhook(int sig)
 /* to be executed on normal or signal-induced termination */
 {
     struct query	*ctl;
@@ -1059,7 +1167,7 @@ void termhook(int sig)
      */
 
     if (sig != 0)
-        error(0, 0, "terminated with signal %d", sig);
+        report(stdout, 0, _("terminated with signal %d"), sig);
     else
 	/* terminate all SMTP connections cleanly */
 	for (ctl = querylist; ctl; ctl = ctl->next)
@@ -1119,8 +1227,8 @@ static int query_host(struct query *ctl)
 	time_t now;
 
 	time(&now);
-	fprintf(stderr, "fetchmail: %s querying %s (protocol %s) at %s",
-	    RELEASE_ID,
+	report(stdout, 0, _("%s querying %s (protocol %s) at %s"),
+	    VERSION,
 	    ctl->server.pollname, showproto(ctl->server.protocol), ctime(&now));
     }
     switch (ctl->server.protocol) {
@@ -1138,7 +1246,7 @@ static int query_host(struct query *ctl)
 #ifdef POP2_ENABLE
 	return(doPOP2(ctl));
 #else
-	fprintf(stderr, "POP2 support is not configured.\n");
+	report(stderr, 0, _("POP2 support is not configured.\n"));
 	return(PS_PROTOCOL);
 #endif /* POP2_ENABLE */
 	break;
@@ -1148,7 +1256,7 @@ static int query_host(struct query *ctl)
 #ifdef POP3_ENABLE
 	return(doPOP3(ctl));
 #else
-	fprintf(stderr, "POP3 support is not configured.\n");
+	report(stderr, 0, _("POP3 support is not configured.\n"));
 	return(PS_PROTOCOL);
 #endif /* POP3_ENABLE */
 	break;
@@ -1160,47 +1268,48 @@ static int query_host(struct query *ctl)
 #ifdef IMAP_ENABLE
 	return(doIMAP(ctl));
 #else
-	fprintf(stderr, "IMAP support is not configured.\n");
+	report(stderr, 0, _("IMAP support is not configured.\n"));
 	return(PS_PROTOCOL);
 #endif /* IMAP_ENABLE */
 	break;
     case P_ETRN:
 #ifndef ETRN_ENABLE
-	fprintf(stderr, "ETRN support is not configured.\n");
+	report(stderr, 0, _("ETRN support is not configured.\n"));
 	return(PS_PROTOCOL);
 #else
 #ifdef HAVE_GETHOSTBYNAME
 	return(doETRN(ctl));
 #else
-	fprintf(stderr, "Cannot support ETRN without gethostbyname(2).\n");
+	report(stderr, 0, _("Cannot support ETRN without gethostbyname(2).\n"));
 	return(PS_PROTOCOL);
 #endif /* HAVE_GETHOSTBYNAME */
 #endif /* ETRN_ENABLE */
     default:
-	error(0, 0, "unsupported protocol selected.");
+	report(stderr, 0, _("unsupported protocol selected."));
 	return(PS_PROTOCOL);
     }
 }
 
-void dump_params (struct runctl *runp, struct query *querylist, flag implicit)
+static void dump_params (struct runctl *runp,
+			 struct query *querylist, flag implicit)
 /* display query parameters in English */
 {
     struct query *ctl;
 
     if (runp->poll_interval)
-	printf("Poll interval is %d seconds\n", runp->poll_interval);
+	printf(_("Poll interval is %d seconds\n"), runp->poll_interval);
     if (runp->logfile)
-	printf("Logfile is %s\n", runp->logfile);
+	printf(_("Logfile is %s\n"), runp->logfile);
     if (strcmp(runp->idfile, IDFILE_NAME))
-	printf("Idfile is %s\n", runp->idfile);
+	printf(_("Idfile is %s\n"), runp->idfile);
 #if defined(HAVE_SYSLOG)
     if (runp->use_syslog)
-	printf("Progress messages will be logged via syslog\n");
+	printf(_("Progress messages will be logged via syslog\n"));
 #endif
     if (runp->invisible)
-	printf("Fetchmail will masquerade and will not generate Received\n");
+	printf(_("Fetchmail will masquerade and will not generate Received\n"));
     if (runp->postmaster)
-	printf("Fetchmail will forward misaddressed multidrop messages to %s.\n",
+	printf(_("Fetchmail will forward misaddressed multidrop messages to %s.\n"),
 	       runp->postmaster);
 
     for (ctl = querylist; ctl; ctl = ctl->next)
@@ -1208,20 +1317,20 @@ void dump_params (struct runctl *runp, struct query *querylist, flag implicit)
 	if (!ctl->active || (implicit && ctl->server.skip))
 	    continue;
 
-	printf("Options for retrieving from %s@%s:\n",
+	printf(_("Options for retrieving from %s@%s:\n"),
 	       ctl->remotename, visbuf(ctl->server.pollname));
 
 	if (ctl->server.via && (ctl->server.protocol != P_ETRN))
-	    printf("  Mail will be retrieved via %s\n", ctl->server.via);
+	    printf(_("  Mail will be retrieved via %s\n"), ctl->server.via);
 
 	if (ctl->server.interval)
-	    printf("  Poll of this server will occur every %d intervals.\n",
+	    printf(_("  Poll of this server will occur every %d intervals.\n"),
 		   ctl->server.interval);
 	if (ctl->server.truename)
-	    printf("  True name of server is %s.\n", ctl->server.truename);
+	    printf(_("  True name of server is %s.\n"), ctl->server.truename);
 	if (ctl->server.skip || outlevel >= O_VERBOSE)
-	    printf("  This host will%s be queried when no host is specified.\n",
-		   ctl->server.skip ? " not" : "");
+	    printf(_("  This host %s be queried when no host is specified.\n"),
+		   ctl->server.skip ? _("will not") : _("will"));
 	/*
 	 * Don't poll for password when there is one or when using the ETRN
 	 * or IMAP-GSS protocol
@@ -1231,14 +1340,14 @@ void dump_params (struct runctl *runp, struct query *querylist, flag implicit)
             && (ctl->server.protocol != P_IMAP_GSS)
 #endif /* GSSAPI */
         )
-	    printf("  Password will be prompted for.\n");
+	    printf(_("  Password will be prompted for.\n"));
 	else if (outlevel >= O_VERBOSE)
 	    if (ctl->server.protocol == P_APOP)
-		printf("  APOP secret = \"%s\".\n", visbuf(ctl->password));
+		printf(_("  APOP secret = \"%s\".\n"), visbuf(ctl->password));
 	    else if (ctl->server.protocol == P_RPOP)
-		printf("  RPOP id = \"%s\".\n", visbuf(ctl->password));
+		printf(_("  RPOP id = \"%s\".\n"), visbuf(ctl->password));
 	    else
-		printf("  Password = \"%s\".\n", visbuf(ctl->password));
+		printf(_("  Password = \"%s\".\n"), visbuf(ctl->password));
 	if (ctl->server.protocol == P_POP3 
 #if INET6
 	    && !strcmp(ctl->server.service, KPOP_PORT)
@@ -1247,119 +1356,121 @@ void dump_params (struct runctl *runp, struct query *querylist, flag implicit)
 #endif /* INET6 */
 	    && (ctl->server.preauthenticate == A_KERBEROS_V4 ||
 		ctl->server.preauthenticate == A_KERBEROS_V5))
-	    printf("  Protocol is KPOP with Kerberos %s authentication",
+	    printf(_("  Protocol is KPOP with Kerberos %s authentication"),
 		   ctl->server.preauthenticate == A_KERBEROS_V5 ? "V" : "IV");
 	else
-	    printf("  Protocol is %s", showproto(ctl->server.protocol));
+	    printf(_("  Protocol is %s"), showproto(ctl->server.protocol));
 #if INET6
 	if (ctl->server.service)
-	    printf(" (using service %s)", ctl->server.service);
+	    printf(_(" (using service %s)"), ctl->server.service);
 	if (ctl->server.netsec)
-	    printf(" (using network security options %s)", ctl->server.netsec);
+	    printf(_(" (using network security options %s)"), ctl->server.netsec);
 #else /* INET6 */
 	if (ctl->server.port)
-	    printf(" (using port %d)", ctl->server.port);
+	    printf(_(" (using port %d)"), ctl->server.port);
 #endif /* INET6 */
 	else if (outlevel >= O_VERBOSE)
-	    printf(" (using default port)");
+	    printf(_(" (using default port)"));
 	if (ctl->server.uidl && (ctl->server.protocol != P_ETRN))
-	    printf(" (forcing UIDL use)");
+	    printf(_(" (forcing UIDL use)"));
 	putchar('.');
 	putchar('\n');
 	if (ctl->server.preauthenticate == A_KERBEROS_V4)
-	    printf("  Kerberos V4 preauthentication enabled.\n");
+	    printf(_("  Kerberos V4 preauthentication enabled.\n"));
 	if (ctl->server.preauthenticate == A_KERBEROS_V5)
-	    printf("  Kerberos V5 preauthentication enabled.\n");
+	    printf(_("  Kerberos V5 preauthentication enabled.\n"));
 	if (ctl->server.timeout > 0)
-	    printf("  Server nonresponse timeout is %d seconds", ctl->server.timeout);
+	    printf(_("  Server nonresponse timeout is %d seconds"), ctl->server.timeout);
 	if (ctl->server.timeout ==  CLIENT_TIMEOUT)
-	    printf(" (default).\n");
+	    printf(_(" (default).\n"));
 	else
 	    printf(".\n");
 
 	if (ctl->server.protocol != P_ETRN) {
 		if (!ctl->mailboxes->id)
-		    printf("  Default mailbox selected.\n");
+		    printf(_("  Default mailbox selected.\n"));
 		else
 		{
 		    struct idlist *idp;
 
-		    printf("  Selected mailboxes are:");
+		    printf(_("  Selected mailboxes are:"));
 		    for (idp = ctl->mailboxes; idp; idp = idp->next)
 			printf(" %s", idp->id);
 		    printf("\n");
 		}
-		printf("  %s messages will be retrieved (--all %s).\n",
-		       ctl->fetchall ? "All" : "Only new",
+		printf(_("  %s messages will be retrieved (--all %s).\n"),
+		       ctl->fetchall ? _("All") : _("Only new"),
 		       ctl->fetchall ? "on" : "off");
-		printf("  Fetched messages will%s be kept on the server (--keep %s).\n",
-		       ctl->keep ? "" : " not",
+		printf(_("  Fetched messages %s be kept on the server (--keep %s).\n"),
+		       ctl->keep ? _("will") : _("will not"),
 		       ctl->keep ? "on" : "off");
-		printf("  Old messages will%s be flushed before message retrieval (--flush %s).\n",
-		       ctl->flush ? "" : " not",
+		printf(_("  Old messages %s be flushed before message retrieval (--flush %s).\n"),
+		       ctl->flush ? _("will") : _("will not"),
 		       ctl->flush ? "on" : "off");
-		printf("  Rewrite of server-local addresses is %sabled (--norewrite %s).\n",
-		       ctl->rewrite ? "en" : "dis",
+		printf(_("  Rewrite of server-local addresses is %s (--norewrite %s).\n"),
+		       ctl->rewrite ? _("enabled") : _("disabled"),
 		       ctl->rewrite ? "off" : "on");
-		printf("  Carriage-return stripping is %sabled (stripcr %s).\n",
-		       ctl->stripcr ? "en" : "dis",
+		printf(_("  Carriage-return stripping is %s (stripcr %s).\n"),
+		       ctl->stripcr ? _("enabled") : _("disabled"),
 		       ctl->stripcr ? "on" : "off");
-		printf("  Carriage-return forcing is %sabled (forcecr %s).\n",
-		       ctl->forcecr ? "en" : "dis",
+		printf(_("  Carriage-return forcing is %s (forcecr %s).\n"),
+		       ctl->forcecr ? _("enabled") : _("disabled"),
 		       ctl->forcecr ? "on" : "off");
-		printf("  Interpretation of Content-Transfer-Encoding is %sabled (pass8bits %s).\n",
-		       ctl->pass8bits ? "dis" : "en",
+		printf(_("  Interpretation of Content-Transfer-Encoding is %s (pass8bits %s).\n"),
+		       ctl->pass8bits ? _("disabled") : _("enabled"),
 		       ctl->pass8bits ? "on" : "off");
-		printf("  MIME decoding is %sabled (mimedecode %s).\n",
-		       ctl->mimedecode ? "en" : "dis",
+		printf(_("  MIME decoding is %s (mimedecode %s).\n"),
+		       ctl->mimedecode ? _("enabled") : _("disabled"),
 		       ctl->mimedecode ? "on" : "off");
-		printf("  Nonempty Status lines will be %s (dropstatus %s)\n",
-		       ctl->dropstatus ? "discarded" : "kept",
+		printf(_("  Nonempty Status lines will be %s (dropstatus %s)\n"),
+		       ctl->dropstatus ? _("discarded") : _("kept"),
 		       ctl->dropstatus ? "on" : "off");
 		if (NUM_NONZERO(ctl->limit))
 		{
 		    if (NUM_NONZERO(ctl->limit))
-			printf("  Message size limit is %d bytes (--limit %d).\n", 
+			printf(_("  Message size limit is %d octets (--limit %d).\n"), 
 			       ctl->limit, ctl->limit);
 		    else if (outlevel >= O_VERBOSE)
-			printf("  No message size limit (--limit 0).\n");
+			printf(_("  No message size limit (--limit 0).\n"));
 		    if (run.poll_interval > 0)
-			printf("  Message size warning interval is %d seconds (--warnings %d).\n", 
+			printf(_("  Message size warning interval is %d seconds (--warnings %d).\n"), 
 			       ctl->warnings, ctl->warnings);
 		    else if (outlevel >= O_VERBOSE)
-			printf("  Size warnings on every poll (--warnings 0).\n");
+			printf(_("  Size warnings on every poll (--warnings 0).\n"));
 		}
 		if (NUM_NONZERO(ctl->fetchlimit))
-		    printf("  Received-message limit is %d (--fetchlimit %d).\n",
+		    printf(_("  Received-message limit is %d (--fetchlimit %d).\n"),
 			   ctl->fetchlimit, ctl->fetchlimit);
 		else if (outlevel >= O_VERBOSE)
-		    printf("  No received-message limit (--fetchlimit 0).\n");
+		    printf(_("  No received-message limit (--fetchlimit 0).\n"));
 		if (NUM_NONZERO(ctl->batchlimit))
-		    printf("  SMTP message batch limit is %d.\n", ctl->batchlimit);
+		    printf(_("  SMTP message batch limit is %d.\n"), ctl->batchlimit);
 		else if (outlevel >= O_VERBOSE)
-		    printf("  No SMTP message batch limit (--batchlimit 0).\n");
+		    printf(_("  No SMTP message batch limit (--batchlimit 0).\n"));
 		if (ctl->server.protocol == P_IMAP)
 		    if (NUM_NONZERO(ctl->expunge))
-			printf("  Deletion interval between expunges is %d (--expunge %d).\n", ctl->expunge, ctl->expunge);
+			printf(_("  Deletion interval between expunges is %d (--expunge %d).\n"), ctl->expunge, ctl->expunge);
 		    else if (outlevel >= O_VERBOSE)
-			printf("  No expunges (--expunge 0).\n");
+			printf(_("  No expunges (--expunge 0).\n"));
 	}
-	if (ctl->mda && (ctl->server.protocol != P_ETRN))
-	    printf("  Messages will be delivered with \"%s\".\n", visbuf(ctl->mda));
+	if (ctl->bsmtp)
+	    printf(_("  Messages will be appended to %s as BSMTP\n"), visbuf(ctl->bsmtp));
+	else if (ctl->mda && (ctl->server.protocol != P_ETRN))
+	    printf(_("  Messages will be delivered with \"%s\".\n"), visbuf(ctl->mda));
 	else
 	{
 	    struct idlist *idp;
 
-	    printf("  Messages will be SMTP-forwarded to:");
+	    printf(_("  Messages will be %cMTP-forwarded to:"), ctl->listener);
 	    for (idp = ctl->smtphunt; idp; idp = idp->next)
 	    {
                 printf(" %s", idp->id);
 		if (!idp->val.status.mark)
-		    printf(" (default)");
+		    printf(_(" (default)"));
 	    }
 	    printf("\n");
 	    if (ctl->smtpaddress)
-		printf("  Host part of MAIL FROM line will be %s\n",
+		printf(_("  Host part of MAIL FROM line will be %s\n"),
 		       ctl->smtpaddress);
 	}
 	if (ctl->server.protocol != P_ETRN)
@@ -1368,27 +1479,27 @@ void dump_params (struct runctl *runp, struct query *querylist, flag implicit)
 		{
 		    struct idlist *idp;
 
-		    printf("  Recognized listener spam block responses are:");
+		    printf(_("  Recognized listener spam block responses are:"));
 		    for (idp = ctl->antispam; idp; idp = idp->next)
 			printf(" %d", idp->val.status.num);
 		    printf("\n");
 		}
 		else if (outlevel >= O_VERBOSE)
-		    printf("  Spam-blocking disabled\n");
+		    printf(_("  Spam-blocking disabled\n"));
 	}
 	if (ctl->preconnect)
-	    printf("  Server connection will be brought up with \"%s\".\n",
+	    printf(_("  Server connection will be brought up with \"%s\".\n"),
 		   visbuf(ctl->preconnect));
 	else if (outlevel >= O_VERBOSE)
-	    printf("  No pre-connection command.\n");
+	    printf(_("  No pre-connection command.\n"));
 	if (ctl->postconnect)
-	    printf("  Server connection will be taken down with \"%s\".\n",
+	    printf(_("  Server connection will be taken down with \"%s\".\n"),
 		   visbuf(ctl->postconnect));
 	else if (outlevel >= O_VERBOSE)
-	    printf("  No post-connection command.\n");
+	    printf(_("  No post-connection command.\n"));
 	if (ctl->server.protocol != P_ETRN) {
 		if (!ctl->localnames)
-		    printf("  No localnames declared for this host.\n");
+		    printf(_("  No localnames declared for this host.\n"));
 		else
 		{
 		    struct idlist *idp;
@@ -1398,11 +1509,11 @@ void dump_params (struct runctl *runp, struct query *querylist, flag implicit)
 			++count;
 
 		    if (count > 1 || ctl->wildcard)
-			printf("  Multi-drop mode: ");
+			printf(_("  Multi-drop mode: "));
 		    else
-			printf("  Single-drop mode: ");
+			printf(_("  Single-drop mode: "));
 
-		    printf("%d local name(s) recognized.\n", count);
+		    printf(_("%d local name(s) recognized.\n"), count);
 		    if (outlevel >= O_VERBOSE)
 		    {
 			for (idp = ctl->localnames; idp; idp = idp->next)
@@ -1416,37 +1527,37 @@ void dump_params (struct runctl *runp, struct query *querylist, flag implicit)
 
 		    if (count > 1 || ctl->wildcard)
 		    {
-			printf("  DNS lookup for multidrop addresses is %sabled.\n",
-			       ctl->server.dns ? "en" : "dis");
+			printf(_("  DNS lookup for multidrop addresses is %s.\n"),
+			       ctl->server.dns ? _("enabled") : _("disabled"));
 			if (ctl->server.dns)
 			{
-			    printf("  Server aliases will be compared with multidrop addresses by ");
+			    printf(_("  Server aliases will be compared with multidrop addresses by "));
 	       		    if (ctl->server.checkalias)
-				printf("IP address.\n");
+				printf(_("IP address.\n"));
 			    else
-				printf("name.\n");
+				printf(_("name.\n"));
 			}
 			if (ctl->server.envelope == STRING_DISABLED)
-			    printf("  Envelope-address routing is disabled\n");
+			    printf(_("  Envelope-address routing is disabled\n"));
 			else
 			{
-			    printf("  Envelope header is assumed to be: %s\n",
-				   ctl->server.envelope ? ctl->server.envelope:"Received");
+			    printf(_("  Envelope header is assumed to be: %s\n"),
+				   ctl->server.envelope ? ctl->server.envelope:_("Received"));
 			    if (ctl->server.envskip > 1 || outlevel >= O_VERBOSE)
-				printf("  Number of envelope header to be parsed: %d\n",
+				printf(_("  Number of envelope header to be parsed: %d\n"),
 				       ctl->server.envskip);
 			    if (ctl->server.qvirtual)
-				printf("  Prefix %s will be removed from user id\n",
+				printf(_("  Prefix %s will be removed from user id\n"),
 				       ctl->server.qvirtual);
 			    else if (outlevel >= O_VERBOSE) 
-				printf("  No prefix stripping\n");
+				printf(_("  No prefix stripping\n"));
 			}
 
 			if (ctl->server.akalist)
 			{
 			    struct idlist *idp;
 
-			    printf("  Predeclared mailserver aliases:");
+			    printf(_("  Predeclared mailserver aliases:"));
 			    for (idp = ctl->server.akalist; idp; idp = idp->next)
 				printf(" %s", idp->id);
 			    putchar('\n');
@@ -1455,7 +1566,7 @@ void dump_params (struct runctl *runp, struct query *querylist, flag implicit)
 			{
 			    struct idlist *idp;
 
-			    printf("  Local domains:");
+			    printf(_("  Local domains:"));
 			    for (idp = ctl->server.localdomains; idp; idp = idp->next)
 				printf(" %s", idp->id);
 			    putchar('\n');
@@ -1465,18 +1576,27 @@ void dump_params (struct runctl *runp, struct query *querylist, flag implicit)
 	}
 #ifdef	linux
 	if (ctl->server.interface)
-	    printf("  Connection must be through interface %s.\n", ctl->server.interface);
+	    printf(_("  Connection must be through interface %s.\n"), ctl->server.interface);
 	else if (outlevel >= O_VERBOSE)
-	    printf("  No interface requirement specified.\n");
+	    printf(_("  No interface requirement specified.\n"));
 	if (ctl->server.monitor)
-	    printf("  Polling loop will monitor %s.\n", ctl->server.monitor);
+	    printf(_("  Polling loop will monitor %s.\n"), ctl->server.monitor);
 	else if (outlevel >= O_VERBOSE)
-	    printf("  No monitor interface specified.\n");
+	    printf(_("  No monitor interface specified.\n"));
 #endif
 
+	if (ctl->server.plugin)
+	    printf(_("  Server connections will be mode via plugin %s (--plugin %s).\n"), ctl->server.plugin, ctl->server.plugin);
+	else if (outlevel >= O_VERBOSE)
+	    printf(_("  No plugin command specified.\n"));
+	if (ctl->server.plugout)
+	    printf(_("  Listener connections will be mode via plugout %s (--plugout %s).\n"), ctl->server.plugout, ctl->server.plugout);
+	else if (outlevel >= O_VERBOSE)
+	    printf(_("  No plugout command specified.\n"));
+
 	if (ctl->server.protocol > P_POP2 && (ctl->server.protocol != P_ETRN))
 	    if (!ctl->oldsaved)
-		printf("  No UIDs saved from this host.\n");
+		printf(_("  No UIDs saved from this host.\n"));
 	    else
 	    {
 		struct idlist *idp;
@@ -1485,14 +1605,14 @@ void dump_params (struct runctl *runp, struct query *querylist, flag implicit)
 		for (idp = ctl->oldsaved; idp; idp = idp->next)
 		    ++count;
 
-		printf("  %d UIDs saved.\n", count);
+		printf(_("  %d UIDs saved.\n"), count);
 		if (outlevel >= O_VERBOSE)
 		    for (idp = ctl->oldsaved; idp; idp = idp->next)
-			fprintf(stderr, "\t%s\n", idp->id);
+			printf("\t%s\n", idp->id);
 	    }
 
 	if (ctl->properties)
-	    printf("  Pass-through properties \"%s\".\n",
+	    printf(_("  Pass-through properties \"%s\".\n"),
 		   visbuf(ctl->properties));
     }
 }
