@@ -17,8 +17,10 @@
 #define		P_ETRN		9
 
 #if INET6
+#define		SMTP_PORT	"smtp"
 #define		KPOP_PORT	"kpop"
 #else /* INET6 */
+#define		SMTP_PORT	25
 #define		KPOP_PORT	1109
 #endif /* INET6 */
 
@@ -64,12 +66,14 @@
 #define		PS_EXCLUDE	8	/* client-side exclusion error */
 #define		PS_LOCKBUSY	9	/* server responded lock busy */
 #define		PS_SMTP         10      /* SMTP error */
-#define		PS_DNS		11	/* fatal DNS error */	
-#define		PS_UNDEFINED	12	/* something I hadn't thought of */
-#define		PS_TRANSIENT	13	/* transient failure (internal use) */
-#define		PS_REFUSED	14	/* mail refused (internal use) */
-#define		PS_RETAINED	15	/* message retained (internal use) */
-#define		PS_TRUNCATED	16	/* headers incomplete (internal use) */
+#define		PS_DNS		11	/* fatal DNS error */
+#define		PS_BSMTP	12	/* output batch could not be opened */
+/* leave space for more codes */
+#define		PS_UNDEFINED	23	/* something I hadn't thought of */
+#define		PS_TRANSIENT	24	/* transient failure (internal use) */
+#define		PS_REFUSED	25	/* mail refused (internal use) */
+#define		PS_RETAINED	26	/* message retained (internal use) */
+#define		PS_TRUNCATED	27	/* headers incomplete (internal use) */
 
 /* output noise level */
 #define         O_SILENT	0	/* mute, max squelch, etc. */
@@ -101,7 +105,7 @@ struct runctl
     int		poll_interval;
     flag	use_syslog;
     flag	invisible;
-    const char	*postmaster;
+    char	*postmaster;
 };
 
 struct idlist
@@ -197,6 +201,8 @@ struct hostdata		/* shared among all user connections to given server */
     struct interface_pair_s *interface_pair;
 #endif /* linux */
 
+    char *plugin,*plugout;
+
     /* computed for internal use */
     const struct method *base_protocol;	/* relevant protocol method table */
     int poll_count;			/* count of polls so far */
@@ -217,11 +223,16 @@ struct query
     char *remotename;		/* remote login name to use */
     char *password;		/* remote password to use */
     struct idlist *mailboxes;	/* list of mailboxes to check */
+
+    /* per-forwarding-target data */
     struct idlist *smtphunt;	/* list of SMTP hosts to try forwarding to */
-    char *smtphost;		/* actual SMTP host to point to */
-    char *smtpaddress;		/* address we want to force in the delivery messages */ 
+    char *smtpaddress;		/* address to force in RCPT TO */ 
     struct idlist *antispam;	/* list of listener's antispam response */
     char *mda;			/* local MDA to pass mail to */
+    char *bsmtp;		/* BSMTP output file */
+    char listener;		/* what's the listener's wire protocol? */
+#define SMTP_MODE	'S'
+#define LMTP_MODE	'L'
     char *preconnect;		/* pre-connection command to execute */
     char *postconnect;		/* post-connection command to execute */
 
@@ -234,7 +245,7 @@ struct query
     flag forcecr;		/* if TRUE, force CRs before LFs in text */
     flag pass8bits;		/* if TRUE, ignore Content-Transfer-Encoding */
     flag dropstatus;		/* if TRUE, drop Status lines in mail */
-    flag mimedecode;		/* if TRUE, decode MIME-coded headers/coded printable*/
+    flag mimedecode;		/* if TRUE, decode MIME-armored messages */
     int	limit;			/* limit size of retrieved messages */
     int warnings;		/* size warning interval */
     int	fetchlimit;		/* max # msgs to get in single poll */
@@ -242,15 +253,16 @@ struct query
     int	expunge;		/* max # msgs to pass between expunges */
     char *properties;		/* passthrough properties for extensions */
 
-    struct idlist *oldsaved, *newsaved;
-
     /* internal use -- per-poll state */
     flag active;		/* should we actually poll this server? */
     const char *destaddr;	/* destination host for this query */
     int errcount;		/* count transient errors in last pass */
+    int wedged;			/* wedged by auth failures or timeouts? */
+    char *smtphost;		/* actual SMTP host we connected to */
     int smtp_socket;		/* socket descriptor for SMTP connection */
     unsigned int uid;		/* UID of user to deliver to */
     struct idlist *skipped;	/* messages skipped on the mail server */
+    struct idlist *oldsaved, *newsaved;
 
     /* internal use -- per-message state */
     int mimemsg;		/* bitmask indicating MIME body-type */
@@ -260,6 +272,15 @@ struct query
     struct query *next;		/* next query control block in chain */
 };
 
+struct msgblk			/* message header parsed for open_sink() */
+{
+    char   		*headers;	/* raw message headers */
+    struct idlist	*recipients;	/* addressees */
+    char		return_path[HOSTLEN + USERNAMELEN + 4]; 
+    int			reallen;
+};
+
+
 /*
  * Numeric option handling.  Numeric option value of zero actually means
  * it's unspecified.  Value less than zero is zero.
@@ -316,16 +337,16 @@ extern const char *fetchmailhost;
 
 /* error.c: Error reporting */
 #if defined(HAVE_STDARG_H)
-void error_init(int foreground);
-void error (int status, int errnum, const char *format, ...);
-void error_build (const char *format, ...);
-void error_complete (int status, int errnum, const char *format, ...);
-void error_at_line (int, int, const char *, unsigned int, const char *, ...);
+void report_init(int foreground);
+void report (FILE *fp, int errnum, const char *format, ...);
+void report_build (FILE *fp, const char *format, ...);
+void report_complete (FILE *fp, int errnum, const char *format, ...);
+void report_at_line (FILE *fp, int, const char *, unsigned int, const char *, ...);
 #else
-void error ();
-void error_build ();
-void error_complete ();
-void error_at_line ();
+void report ();
+void report_build ();
+void report_complete ();
+void report_at_line ();
 #endif
 
 /* driver.c: transaction support */
@@ -349,15 +370,22 @@ int gen_transact ();
 extern int phase;
 
 /* mark values for name lists */
-#define XMIT_ACCEPT		1
-#define XMIT_REJECT		2
-#define XMIT_ANTISPAM		3
+#define XMIT_ACCEPT	1	/* accepted; matches local domain or name */
+#define XMIT_REJECT	2	/* rejected; no match */
+#define XMIT_RCPTBAD	3	/* SMTP listener rejected the name */ 
 
 /* sink.c: forwarding */
 int stuffline(struct query *, char *);
-int open_sink(struct query*, const char*, struct idlist*, long reallen, int*, int*);
+int open_sink(struct query*, struct msgblk *, int*, int*);
 void release_sink(struct query *);
-int close_sink(struct query *, flag);
+int close_sink(struct query *, struct msgblk *, flag);
+int open_warning_by_mail(struct query *, struct msgblk *);
+#if defined(HAVE_STDARG_H)
+void stuff_warning(struct query *, const char *, ... );
+#else
+void stuff_warning();
+#endif
+void close_warning_by_mail(struct query *, struct msgblk *);
 
 /* rfc822.c: RFC822 header parsing */
 char *reply_hack(char *, const char *);
@@ -421,7 +449,7 @@ char *xstrdup(const char *);
  #pragma alloca
 #endif
 #endif
-#define	xalloca(ptr, t, n)	if (!(ptr = (t) alloca(n))) error(PS_UNDEFINED, 0, "alloca failed")
+#define	xalloca(ptr, t, n)	if (!(ptr = (t) alloca(n))) report(stderr, PS_UNDEFINED, 0, "alloca failed")
 
 /* protocol driver and methods */
 int do_protocol(struct query *, const struct method *);
@@ -443,6 +471,7 @@ const char *showproto(int);
 void dump_config(struct runctl *runp, struct query *querylist);
 int is_host_alias(const char *, struct query *);
 char *host_fqdn(void);
+char *rfc822timestamp(void);
 #ifdef SDPS_ENABLE
 char *sdps_envto;
 #endif /* SDPS_ENABLE */
@@ -461,9 +490,21 @@ void itimerthread(void*);
    find all of our lock files and stuff. */
 #define getcwd _getcwd2
 #define chdir _chdir2
-#endif
+#endif /* _EMX_ */
+
+# if HAVE_STRERROR
+#  ifndef strerror		/* On some systems, strerror is a macro */
+char *strerror ();
+#  endif
+# endif /* HAVE_STRERROR */
 
 #define STRING_DISABLED	(char *)-1
 #define STRING_DUMMY	""
 
+#ifdef NeXT
+#ifndef S_IXGRP
+#define S_IXGRP 0000010
+#endif
+#endif
+
 /* fetchmail.h ends here */
