@@ -9,7 +9,7 @@
   description: 	getpass() replacement which allows for long passwords.
                 This version hacked by Wilfred Teiken, allowing the
                 password to be piped to fetchmail.
-
+ 
  ***********************************************************************/
 
 #include "config.h"
@@ -21,6 +21,7 @@
 #include <unistd.h>
 #endif
 #include "fetchmail.h"
+#include "i18n.h"
 
 #define INPUT_BUF_SIZE	PASSWORDLEN
 
@@ -67,7 +68,7 @@ char *prompt;
     char *getpass();
     return getpass(prompt);
 #else
-    fputs("ERROR: no support for getpassword() routine\n",stderr);
+    fputs(_("ERROR: no support for getpassword() routine\n"),stderr);
     exit(1);
 #endif
 #else
@@ -189,7 +190,7 @@ static void restore_tty_state(void)
 static RETSIGTYPE sigint_handler(int signum)
 {
     restore_tty_state();
-    error(1, 0, "\nCaught signal... bailing out.");
+    report(stderr, 1, 0, _("\nCaught signal... bailing out."));
 }
 
 /* getpass.c ends here */
