@@ -29,6 +29,7 @@
 #include <net/if.h>
 #include "config.h"
 #include "fetchmail.h"
+#include "i18n.h"
 
 typedef struct {
 	struct in_addr addr, dstaddr, netmask;
@@ -156,7 +157,7 @@ static int inet_aton(const char *cp, struct in_addr *inp) {
     memcpy(inp, &addr, sizeof(addr));
     return 1;
 }
-#endif HAVE_INET_ATON
+#endif /* HAVE_INET_ATON */
 
 void interface_parse(char *buf, struct hostdata *hp)
 /* parse 'interface' specification */
@@ -167,7 +168,7 @@ void interface_parse(char *buf, struct hostdata *hp)
 
 	/* find and isolate just the IP address */
 	if (!(cp1 = strchr(buf, '/')))
-		(void) error(PS_SYNTAX, 0, "missing IP interface address");
+		(void) report(stderr, PS_SYNTAX, 0, _("missing IP interface address"));
 	*cp1++ = '\000';
 
 	/* find and isolate just the netmask */
@@ -179,9 +180,9 @@ void interface_parse(char *buf, struct hostdata *hp)
 	/* convert IP address and netmask */
 	hp->interface_pair = (struct interface_pair_s *)xmalloc(sizeof(struct interface_pair_s));
 	if (!inet_aton(cp1, &hp->interface_pair->interface_address))
-		(void) error(PS_SYNTAX, 0, "invalid IP interface address");
+		(void) report(stderr, PS_SYNTAX, 0, _("invalid IP interface address"));
 	if (!inet_aton(cp2, &hp->interface_pair->interface_mask))
-		(void) error(PS_SYNTAX, 0, "invalid IP interface mask");
+		(void) report(stderr, PS_SYNTAX, 0, _("invalid IP interface mask"));
 	/* apply the mask now to the IP address (range) required */
 	hp->interface_pair->interface_address.s_addr &=
 		hp->interface_pair->interface_mask.s_addr;
@@ -208,7 +209,7 @@ void interface_note_activity(struct hostdata *hp)
 		   count has now changed and they need to be re-updated)
 		*/
 		for (ctl = querylist; ctl; ctl = ctl->next) {
-			if (!strcmp(hp->monitor, ctl->server.monitor))
+			if (ctl->server.monitor && !strcmp(hp->monitor, ctl->server.monitor))
 				ctl->server.monitor_io =
 					ifinfo.rx_packets + ifinfo.tx_packets;
 			/* do NOT update host entries following this one */
@@ -217,7 +218,7 @@ void interface_note_activity(struct hostdata *hp)
 		}
 
 #ifdef	ACTIVITY_DEBUG
-	(void) error(0, 0, "activity on %s -noted- as %d", 
+	(void) report(stdout, 0, _("activity on %s -noted- as %d"), 
 		hp->monitor, hp->monitor_io);
 #endif
 }
@@ -231,7 +232,7 @@ int interface_approve(struct hostdata *hp)
 	if (hp->interface) {
 		/* get interface info */
 		if (!get_ifinfo(hp->interface, &ifinfo)) {
-			(void) error(0, 0, "skipping poll of %s, %s down",
+			(void) report(stdout, 0, _("skipping poll of %s, %s down"),
 				hp->pollname, hp->interface);
 			return(FALSE);
 		}
@@ -239,8 +240,8 @@ int interface_approve(struct hostdata *hp)
 		if ((ifinfo.addr.s_addr &
 				hp->interface_pair->interface_mask.s_addr) !=
 				hp->interface_pair->interface_address.s_addr) {
-			(void) error(0, 0,
-				"skipping poll of %s, %s IP address excluded",
+			(void) report(stdout, 0,
+				_("skipping poll of %s, %s IP address excluded"),
 				hp->pollname, hp->interface);
 			return(FALSE);
 		}
@@ -251,14 +252,14 @@ int interface_approve(struct hostdata *hp)
 		return(TRUE);
 
 #ifdef	ACTIVITY_DEBUG
-	(void) error(0, 0, "activity on %s checked as %d", 
+	(void) report(stdout, 0, _("activity on %s checked as %d"), 
 		hp->monitor, hp->monitor_io);
 #endif
 	/* if monitoring, check link for activity if it is up */
 	if (get_ifinfo(hp->monitor, &ifinfo) &&
 			hp->monitor_io == ifinfo.rx_packets +
 				ifinfo.tx_packets) {
-		(void) error(0, 0, "skipping poll of %s, %s inactive",
+		(void) report(stdout, 0, _("skipping poll of %s, %s inactive"),
 			hp->pollname, hp->monitor);
 		return(FALSE);
 	}
