@@ -4,7 +4,7 @@
 #
 $version = $ARGV[0];
 $size = substr($ARGV[1], 0, 3);
-$wwwhost = $ENV{'WWWHOST'};
+$wwwhost = $ENV{'WWWVIRTUAL'};
 
 @months
   = ('JAN','FEB','MAR','APR','MAY','JUN','JUL','AUG','SEP','OCT','NOV','DEC');
