@@ -1,22 +1,10 @@
 /* netrc.c -- parse the .netrc file to get hosts, accounts, and passwords
-   Copyright (C) 1996, Free Software Foundation, Inc.
+
    Gordon Matzigkeit <gord@gnu.ai.mit.edu>, 1996
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   For license terms, see the file COPYING in this directory.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
-
-/* Compile with -DSTANDALONE to test this module. */
+   Compile with -DSTANDALONE to test this module. */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -26,6 +14,7 @@
 #include "config.h"
 #include "fetchmail.h"
 #include "netrc.h"
+#include "i18n.h"
 
 #ifdef STANDALONE
 /* Normally defined in xstrdup.c. */
@@ -112,12 +101,11 @@ parse_netrc (file)
     /* While there are lines in the file... */
     while (fgets(buf, POPBUFSIZE, fp))
     {
-	ln ++;
+	ln++;
 
 	/* Strip trailing CRLF */
-	p = buf + strlen(buf) - 1;
-	while (*p && isspace(*p))
-	    *p-- = '\0';
+	for (p = buf + strlen(buf) - 1; (p >= buf) && isspace(*p); p--)
+	    *p = '\0';
 
 	/* Parse the line. */
 	p = buf;
@@ -221,12 +209,12 @@ parse_netrc (file)
 	    if (premature_token)
 	    {
 #ifdef HAVE_ERROR
-		error_at_line (0, 0, file, ln,
+		error_at_line (0, file, ln,
 			       _("warning: found \"%s\" before any host names"),
 			       premature_token);
 #else
 		fprintf (stderr,
-			 "%s:%d: warning: found \"%s\" before any host names\n",
+			 _("%s:%d: warning: found \"%s\" before any host names\n"),
 			 file, ln, premature_token);
 #endif
 		premature_token = NULL;
@@ -257,12 +245,15 @@ parse_netrc (file)
 		else if (!strcmp (tok, "password"))
 		    last_token = tok_password;
 
+		else if (!strcmp (tok, "passwd"))
+		    last_token = tok_password;
+
 		else if (!strcmp (tok, "account"))
 		    last_token = tok_account;
 
 		else
 		{
-		    fprintf (stderr, "%s:%d: warning: unknown token \"%s\"\n",
+		    fprintf (stderr, _("%s:%d: warning: unknown token \"%s\"\n"),
 			     file, ln, tok);
 		}
 	    }
