@@ -1,6 +1,7 @@
 /*
  * options.c -- command-line option processing
  *
+ * Copyright 1998 by Eric S. Raymond.
  * For license terms, see the file COPYING in this directory.
  */
 
@@ -19,6 +20,7 @@
 
 #include "getopt.h"
 #include "fetchmail.h"
+#include "i18n.h"
 
 #define LA_HELP		1
 #define LA_VERSION	2 
@@ -58,11 +60,15 @@
 #define LA_FETCHLIMIT	36
 #define LA_EXPUNGE	37
 #define LA_MDA		38
-#define LA_NETSEC	39
-#define LA_INTERFACE    40
-#define LA_MONITOR      41
-#define LA_CONFIGDUMP	42
-#define LA_YYDEBUG	43
+#define LA_BSMTP	39
+#define LA_LMTP		40
+#define LA_PLUGIN	41
+#define LA_PLUGOUT	42
+#define LA_NETSEC	43
+#define LA_INTERFACE    44
+#define LA_MONITOR      45
+#define LA_CONFIGDUMP	46
+#define LA_YYDEBUG	47
 
 /* options still left: CDgGhHjJoORwWxXYz */
 static const char *shortoptions = 
@@ -71,10 +77,10 @@ static const char *shortoptions =
 static const struct option longoptions[] = {
 /* this can be const because all flag fields are 0 and will never get set */
   {"help",	no_argument,	   (int *) 0, LA_HELP        },
-  {"version",   no_argument,       (int *) 0, LA_VERSION     },
+  {"version",	no_argument,       (int *) 0, LA_VERSION     },
   {"check",	no_argument,	   (int *) 0, LA_CHECK       },
-  {"silent",    no_argument,       (int *) 0, LA_SILENT      },
-  {"verbose",   no_argument,       (int *) 0, LA_VERBOSE     },
+  {"silent",	no_argument,       (int *) 0, LA_SILENT      },
+  {"verbose",	no_argument,       (int *) 0, LA_VERBOSE     },
   {"daemon",	required_argument, (int *) 0, LA_DAEMON      },
   {"nodetach",	no_argument,	   (int *) 0, LA_NODETACH    },
   {"quit",	no_argument,	   (int *) 0, LA_QUIT        },
@@ -96,34 +102,38 @@ static const struct option longoptions[] = {
   {"qvirtual",	required_argument, (int *) 0, LA_QVIRTUAL    },
 
   {"user",	required_argument, (int *) 0, LA_USERNAME    },
-  {"username",  required_argument, (int *) 0, LA_USERNAME    },
+  {"username",	required_argument, (int *) 0, LA_USERNAME    },
 
   {"all",	no_argument,       (int *) 0, LA_ALL         },
   {"nokeep",	no_argument,	   (int *) 0, LA_NOKEEP      },
-  {"keep",      no_argument,       (int *) 0, LA_KEEP        },
+  {"keep",	no_argument,       (int *) 0, LA_KEEP        },
   {"flush",	no_argument,	   (int *) 0, LA_FLUSH       },
   {"norewrite",	no_argument,	   (int *) 0, LA_NOREWRITE   },
   {"limit",	required_argument, (int *) 0, LA_LIMIT       },
   {"warnings",	required_argument, (int *) 0, LA_WARNINGS    },
 
-  {"folder",    required_argument, (int *) 0, LA_FOLDER	     },
+  {"folder",	required_argument, (int *) 0, LA_FOLDER	     },
   {"smtphost",	required_argument, (int *) 0, LA_SMTPHOST    },
   {"smtpaddress", required_argument, (int *) 0, LA_SMTPADDR  },
   {"antispam",	required_argument, (int *) 0, LA_ANTISPAM    },
   
   {"batchlimit",required_argument, (int *) 0, LA_BATCHLIMIT  },
   {"fetchlimit",required_argument, (int *) 0, LA_FETCHLIMIT  },
-  {"expunge",   required_argument, (int *) 0, LA_EXPUNGE     },
+  {"expunge",	required_argument, (int *) 0, LA_EXPUNGE     },
   {"mda",	required_argument, (int *) 0, LA_MDA         },
+  {"bsmtp",	required_argument, (int *) 0, LA_BSMTP       },
+  {"lmtp",	no_argument,       (int *) 0, LA_LMTP        },
 
 #ifdef INET6
-  {"netsec",	required_argument, (int *) 0, LA_NETSEC    },
+  {"netsec",	required_argument, (int *) 0, LA_NETSEC      },
 #endif /* INET6 */
 
 #if defined(linux) && !INET6
   {"interface",	required_argument, (int *) 0, LA_INTERFACE   },
   {"monitor",	required_argument, (int *) 0, LA_MONITOR     },
 #endif /* defined(linux) && !INET6 */
+  {"plugin",	required_argument, (int *) 0, LA_PLUGIN      },
+  {"plugout",	required_argument, (int *) 0, LA_PLUGOUT     },
 
   {"configdump",no_argument,	   (int *) 0, LA_CONFIGDUMP  },
 
@@ -149,7 +159,7 @@ static int xatoi(char *s, int *errflagptr)
 
     /* any invalid chars in string? */
     if ( (endptr == s) || (*endptr != '\0') ) {
-    	(void) fprintf(stderr, "String '%s' is not a valid number string.\n", s);
+    	(void) fprintf(stderr, _("String '%s' is not a valid number string.\n"), s);
 	(*errflagptr)++;
 	return 0;
     }
@@ -158,8 +168,8 @@ static int xatoi(char *s, int *errflagptr)
     if ( (((value == LONG_MAX) || (value == LONG_MIN)) && (errno == ERANGE)) ||
 				(value > INT_MAX) || (value < INT_MIN)) {
 
-    	(void) fprintf(stderr, "Value of string '%s' is %s than %d.\n", s,
-					(value < 0) ? "smaller": "larger",
+    	(void) fprintf(stderr, _("Value of string '%s' is %s than %d.\n"), s,
+					(value < 0) ? _("smaller"): _("larger"),
 					(value < 0) ? INT_MIN : INT_MAX);
 	(*errflagptr)++;
 	return 0;
@@ -193,7 +203,7 @@ static int xatoi(char *s, int *errflagptr)
 
     /* check for trailing garbage */
     if (i != len) {
-    	(void) fprintf(stderr, "String '%s' is not a valid number string.\n", s);
+    	(void) fprintf(stderr, _("String '%s' is not a valid number string.\n"), s);
     	(*errflagptr)++;
 	return 0;
     }
@@ -219,8 +229,9 @@ struct query *ctl;	/* option record to be initialized */
      */
 
     int c;
-    int ocount = 0;     /* count of destinations specified */
-    int errflag = 0;   /* TRUE when a syntax error is detected */
+    int ocount = 0;	/* count of destinations specified */
+    int errflag = 0;	/* TRUE when a syntax error is detected */
+    int helpflag = 0;	/* TRUE when option help was explicitly requested */
     int option_index;
     char *buf, *cp;
 
@@ -284,6 +295,7 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case LA_POSTMASTER:
 	    rctl->postmaster = (char *) xmalloc(strlen(optarg)+1);
+	    strcpy(rctl->postmaster,optarg);
 	    break;
 	case 'p':
 	case LA_PROTOCOL:
@@ -332,7 +344,7 @@ struct query *ctl;	/* option record to be initialized */
 	    else if (strcasecmp(optarg,"etrn") == 0)
 		ctl->server.protocol = P_ETRN;
 	    else {
-		fprintf(stderr,"Invalid protocol `%s' specified.\n", optarg);
+		fprintf(stderr,_("Invalid protocol `%s' specified.\n"), optarg);
 		errflag++;
 	    }
 	    break;
@@ -355,15 +367,15 @@ struct query *ctl;	/* option record to be initialized */
 	    else if (strcmp(optarg, "kerberos") == 0)
 #ifdef KERBEROS_V5
 		ctl->server.preauthenticate = A_KERBEROS_V5;
-	    else if (strcmp(optarg, "kerberos_v5") == 0)
-		ctl->server.preauthenticate = A_KERBEROS_V5;
 #else
 		ctl->server.preauthenticate = A_KERBEROS_V4;
+#endif /* KERBEROS_V5 */
+	    else if (strcmp(optarg, "kerberos_v5") == 0)
+		ctl->server.preauthenticate = A_KERBEROS_V5;
 	    else if (strcmp(optarg, "kerberos_v4") == 0)
 		ctl->server.preauthenticate = A_KERBEROS_V4;
-#endif /* KERBEROS_V5 */
 	    else {
-		fprintf(stderr,"Invalid preauthentication `%s' specified.\n", optarg);
+		fprintf(stderr,_("Invalid preauthentication `%s' specified.\n"), optarg);
 		errflag++;
 	    }
 	    break;
@@ -468,13 +480,20 @@ struct query *ctl;	/* option record to be initialized */
 	    ctl->mda = xstrdup(optarg);
 	    ocount++;
 	    break;
+	case LA_BSMTP:
+	    ctl->bsmtp = xstrdup(optarg);
+	    ocount++;
+	    break;
+	case LA_LMTP:
+	    ctl->listener = LMTP_MODE;
+	    break;
 
 	case 'T':
 	case LA_NETSEC:
 #if NET_SECURITY
 	    ctl->server.netsec = (void *)optarg;
 #else
-	    fprintf(stderr, "fetchmail: network security support is disabled\n");
+	    fprintf(stderr, _("fetchmail: network security support is disabled\n"));
 	    errflag++;
 #endif /* NET_SECURITY */
 	    break;
@@ -489,6 +508,12 @@ struct query *ctl;	/* option record to be initialized */
 	    ctl->server.monitor = xstrdup(optarg);
 	    break;
 #endif /* defined(linux) && !INET6 */
+	case LA_PLUGIN:
+	    ctl->server.plugin = xstrdup(optarg);
+	    break;
+	case LA_PLUGOUT:
+	    ctl->server.plugout = xstrdup(optarg);
+	    break;
 
 	case 'y':
 	case LA_YYDEBUG:
@@ -516,64 +541,73 @@ struct query *ctl;	/* option record to be initialized */
 	case '?':
 	case LA_HELP:
 	default:
-	    errflag++;
+	    helpflag++;
 	}
     }
 
-    if (errflag || ocount > 1) {
+    if (errflag || ocount > 1 || helpflag) {
 	/* squawk if syntax errors were detected */
-#define P(s)	fputs(s, stderr)
-	P("usage:  fetchmail [options] [server ...]\n");
-	P("  Options are as follows:\n");
-	P("  -?, --help        display this option help\n");
-	P("  -V, --version     display version info\n");
+#define P(s)	fputs(s, helpflag ? stdout : stderr)
+	P(_("usage:  fetchmail [options] [server ...]\n"));
+	P(_("  Options are as follows:\n"));
+	P(_("  -?, --help        display this option help\n"));
+	P(_("  -V, --version     display version info\n"));
 
-	P("  -c, --check       check for messages without fetching\n");
-	P("  -s, --silent      work silently\n");
-	P("  -v, --verbose     work noisily (diagnostic output)\n");
-	P("  -d, --daemon      run as a daemon once per n seconds\n");
-	P("  -N, --nodetach    don't detach daemon process\n");
-	P("  -q, --quit        kill daemon process\n");
-	P("  -L, --logfile     specify logfile name\n");
-	P("      --syslog      use syslog(3) for most messages when running as a daemon\n");
-	P("      --invisible   don't write Received & enable host spoofing\n");
-	P("  -f, --fetchmailrc specify alternate run control file\n");
-	P("  -i, --idfile      specify alternate UIDs file\n");
-	P("      --postmaster  specify recipient of last resort\n");
+	P(_("  -c, --check       check for messages without fetching\n"));
+	P(_("  -s, --silent      work silently\n"));
+	P(_("  -v, --verbose     work noisily (diagnostic output)\n"));
+	P(_("  -d, --daemon      run as a daemon once per n seconds\n"));
+	P(_("  -N, --nodetach    don't detach daemon process\n"));
+	P(_("  -q, --quit        kill daemon process\n"));
+	P(_("  -L, --logfile     specify logfile name\n"));
+	P(_("      --syslog      use syslog(3) for most messages when running as a daemon\n"));
+	P(_("      --invisible   don't write Received & enable host spoofing\n"));
+	P(_("  -f, --fetchmailrc specify alternate run control file\n"));
+	P(_("  -i, --idfile      specify alternate UIDs file\n"));
+	P(_("      --postmaster  specify recipient of last resort\n"));
 #if defined(linux) && !INET6
-	P("  -I, --interface   interface required specification\n");
-	P("  -M, --monitor     monitor interface for activity\n");
+	P(_("  -I, --interface   interface required specification\n"));
+	P(_("  -M, --monitor     monitor interface for activity\n"));
 #endif
+	P(_("      --plugin      specify external command to open connection\n"));
+	P(_("      --plugout     specify external command to open smtp connection\n"));
 
-	P("  -p, --protocol    specify retrieval protocol (see man page)\n");
-	P("  -U, --uidl        force the use of UIDLs (pop3 only)\n");
-	P("  -P, --port        TCP/IP service port to connect to\n");
-	P("  -A, --auth        authentication type (password or kerberos)\n");
-	P("  -t, --timeout     server nonresponse timeout\n");
-	P("  -E, --envelope    envelope address header\n");
-	P("  -Q, --qvirtual    prefix to remove from local user id\n");
+	P(_("  -p, --protocol    specify retrieval protocol (see man page)\n"));
+	P(_("  -U, --uidl        force the use of UIDLs (pop3 only)\n"));
+	P(_("  -P, --port        TCP/IP service port to connect to\n"));
+	P(_("  -A, --auth        authentication type (password or kerberos)\n"));
+	P(_("  -t, --timeout     server nonresponse timeout\n"));
+	P(_("  -E, --envelope    envelope address header\n"));
+	P(_("  -Q, --qvirtual    prefix to remove from local user id\n"));
 
-	P("  -u, --username    specify users's login on server\n");
-	P("  -a, --all         retrieve old and new messages\n");
-	P("  -K, --nokeep      delete new messages after retrieval\n");
-	P("  -k, --keep        save new messages after retrieval\n");
-	P("  -F, --flush       delete old messages from server\n");
-	P("  -n, --norewrite   don't rewrite header addresses\n");
-	P("  -l, --limit       don't fetch messages over given size\n");
-	P("  -w, --warnings   interval between warning mail notification\n");
+	P(_("  -u, --username    specify users's login on server\n"));
+	P(_("  -a, --all         retrieve old and new messages\n"));
+	P(_("  -K, --nokeep      delete new messages after retrieval\n"));
+	P(_("  -k, --keep        save new messages after retrieval\n"));
+	P(_("  -F, --flush       delete old messages from server\n"));
+	P(_("  -n, --norewrite   don't rewrite header addresses\n"));
+	P(_("  -l, --limit       don't fetch messages over given size\n"));
+	P(_("  -w, --warnings    interval between warning mail notification\n"));
 
 #if NET_SECURITY
-	P("  -T, --netsec      set IP security request\n");
+	P(_("  -T, --netsec      set IP security request\n"));
 #endif /* NET_SECURITY */
-	P("  -S, --smtphost    set SMTP forwarding host\n");
-	P("  -D, --smtpaddress set SMTP delivery domain to use\n");
-	P("  -Z, --antispam,   set antispam response values\n");
-	P("  -b, --batchlimit  set batch limit for SMTP connections\n");
-	P("  -B, --fetchlimit  set fetch limit for server connections\n");
-	P("  -e, --expunge     set max deletions between expunges\n");
-	P("  -r, --folder      specify remote folder name\n");
+	P(_("  -S, --smtphost    set SMTP forwarding host\n"));
+	P(_("  -D, --smtpaddress set SMTP delivery domain to use\n"));
+	P(_("  -Z, --antispam,   set antispam response values\n"));
+	P(_("  -b, --batchlimit  set batch limit for SMTP connections\n"));
+	P(_("  -B, --fetchlimit  set fetch limit for server connections\n"));
+	P(_("  -e, --expunge     set max deletions between expunges\n"));
+        P(_("      --mda         set MDA to use for forwarding\n"));
+        P(_("      --bsmtp       set output BSMTP file\n"));
+        P(_("      --lmtp        use LMTP (RFC2033) for delivery\n"));
+	P(_("  -r, --folder      specify remote folder name\n"));
 #undef P
-	return(-1);
+
+	if (helpflag)
+	    exit(PS_SUCCESS);
+	else
+	    exit(PS_SYNTAX);
     }
 
     return(optind);
