@@ -1,6 +1,7 @@
 /*
  * pop3.c -- POP3 protocol methods
  *
+ * Copyright 1998 by Eric S. Raymond.
  * For license terms, see the file COPYING in this directory.
  */
 
@@ -18,6 +19,7 @@
  
 #include  "fetchmail.h"
 #include  "socket.h"
+#include  "i18n.h"
 
 #if OPIE
 #include <opie.h>
@@ -90,7 +92,7 @@ int pop3_ok (int sock, char *argbuf)
 	    else
 		ok = PS_AUTHFAIL;
 	    if (*bufp)
-	      error(0,0,bufp);
+	      report(stderr, 0, bufp);
 	}
 	else
 	    ok = PS_PROTOCOL;
@@ -177,7 +179,7 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	  i = opiegenerator(challenge, !strcmp(ctl->password, "opie") ? "" : ctl->password, response);
 	  if ((i == -2) && !run.poll_interval) {
 	    char secret[OPIE_SECRET_MAX+1];
-	    fprintf(stderr, "Secret pass phrase: ");
+	    fprintf(stderr, _("Secret pass phrase: "));
 	    if (opiereadpass(secret, sizeof(secret), 0))
 	      i = opiegenerator(challenge,  secret, response);
 	    memset(secret, 0, sizeof(secret));
@@ -203,7 +205,7 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	for (start = greeting;  *start != 0 && *start != '<';  start++)
 	    continue;
 	if (*start == 0) {
-	    error(0, -1, "Required APOP timestamp not found in greeting");
+	    report(stderr, 0, _("Required APOP timestamp not found in greeting"));
 	    return(PS_AUTHFAIL);
 	}
 
@@ -211,7 +213,7 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	for (end = start;  *end != 0  && *end != '>';  end++)
 	    continue;
 	if (*end == 0 || end == start + 1) {
-	    error(0, -1, "Timestamp syntax error in greeting");
+	    report(stderr, 0, _("Timestamp syntax error in greeting"));
 	    return(PS_AUTHFAIL);
 	}
 	else
@@ -222,7 +224,7 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	strcpy(msg,start);
 	strcat(msg,ctl->password);
 
-	strcpy(ctl->digest, MD5Digest(msg));
+	strcpy(ctl->digest, MD5Digest((unsigned char *)msg));
 
 	ok = gen_transact(sock, "APOP %s %s", ctl->remotename, ctl->digest);
 	break;
@@ -233,7 +235,7 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	break;
 
     default:
-	error(0, 0, "Undefined protocol request in POP3_auth");
+	report(stderr, 0, _("Undefined protocol request in POP3_auth"));
 	ok = PS_ERROR;
     }
 
@@ -241,7 +243,7 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
     {
 	/* maybe we detected a lock-busy condition? */
         if (ok == PS_LOCKBUSY)
-	    error(0, 0, "lock busy!  Is another session active?"); 
+	    report(stderr, 0, _("lock busy!  Is another session active?")); 
 
 	return(ok);
     }
@@ -349,8 +351,8 @@ pop3_slowuidl( int sock,  struct query *ctl, int *countp, int *newp)
 		    try_id--;
 		}
 	    } else {
-		error(0,0,"Messages inserted into list on server. "
-		      "Cannot handle this.");
+		report(stderr, 0, 
+		       _("Messages inserted into list on server. Cannot handle this."));
 		return -1;
 	    }
 	} 
@@ -423,7 +425,7 @@ static int pop3_getrange(int sock,
 	{
 	    if (sscanf(buf, "%d", &last) == 0)
 	    {
-		error(0, 0, "protocol error");
+		report(stderr, 0, _("protocol error"));
 		return(PS_ERROR);
 	    }
 	    *newp = (*countp - last);
@@ -436,7 +438,7 @@ static int pop3_getrange(int sock,
 		/* don't worry, yet! do it the slow way */
 		if((ok = pop3_slowuidl( sock, ctl, countp, newp))!=0)
 		{
-		    error(0, 0, "protocol error while fetching UIDLs");
+		    report(stderr, 0, _("protocol error while fetching UIDLs"));
 		    return(PS_ERROR);
 		}
 	    }
@@ -561,7 +563,7 @@ static int pop3_fetch(int sock, struct query *ctl, int number, int *lenp)
             if (linecount == 5)
             {
                 /* Wrap address with To: <> so nxtaddr() likes it */
-                sdps_envto = malloc(strlen(buf)+7);
+                sdps_envto = xmalloc(strlen(buf)+7);
                 sprintf(sdps_envto,"To: <%s>",buf);
             }
 	} while
@@ -660,11 +662,11 @@ int doPOP3 (struct query *ctl)
 {
 #ifndef MBOX
     if (ctl->mailboxes->id) {
-	fprintf(stderr,"Option --remote is not supported with POP3\n");
+	fprintf(stderr,_("Option --remote is not supported with POP3\n"));
 	return(PS_SYNTAX);
     }
 #endif /* MBOX */
-    peek_capable = FALSE;
+    peek_capable = !ctl->fetchall;
     return(do_protocol(ctl, &pop3));
 }
 #endif /* POP3_ENABLE */
