@@ -29,7 +29,9 @@ const char *host;	/* server hostname */
 {
     char *from, *cp, last_nws = '\0', *parens_from = NULL;
     int parendepth, state, has_bare_name_part, has_host_part;
+#ifndef TESTMAIN
     int addresscount = 1;
+#endif /* TESTMAIN */
 
     if (strncasecmp("From: ", buf, 6)
 	&& strncasecmp("To: ", buf, 4)
@@ -51,8 +53,8 @@ const char *host;	/* server hostname */
 
 #ifndef TESTMAIN
     if (outlevel >= O_DEBUG)
-	error_build(0, 0, "About to rewrite %s", buf);
-#else
+	report_build(stdout, "About to rewrite %s", buf);
+
     /* make room to hack the address; buf must be malloced */
     for (cp = buf; *cp; cp++)
 	if (*cp == ',' || isspace(*cp))
@@ -176,7 +178,7 @@ const char *host;	/* server hostname */
 
 #ifndef TESTMAIN
     if (outlevel >= O_DEBUG)
-	error_complete(0, 0, "Rewritten version is %s", buf);
+	report_complete(stdout, 0, "Rewritten version is %s", buf);
 #endif /* TESTMAIN */
     return(buf);
 }
