@@ -1,6 +1,7 @@
 /*
  * xmalloc.c -- allocate space or die 
  *
+ * Copyright 1998 by Eric S. Raymond.
  * For license terms, see the file COPYING in this directory.
  */
 
@@ -12,6 +13,7 @@
 #include  <stdlib.h>
 #endif
 #include "fetchmail.h"
+#include "i18n.h"
 
 #if defined(HAVE_VOIDPOINTER)
 #define XMALLOCTYPE void
@@ -26,7 +28,10 @@ xmalloc (int n)
 
     p = (XMALLOCTYPE *) malloc(n);
     if (p == (XMALLOCTYPE *) 0)
-	error(PS_UNDEFINED, errno, "malloc failed");
+    {
+	report(stderr, errno, _("malloc failed"));
+	exit(PS_UNDEFINED);
+    }
     return(p);
 }
 
@@ -37,7 +42,10 @@ xrealloc (XMALLOCTYPE *p, int n)
 	return xmalloc (n);
     p = (XMALLOCTYPE *) realloc(p, n);
     if (p == (XMALLOCTYPE *) 0)
-	error(PS_UNDEFINED, errno, "realloc failed");
+    {
+	report(stderr, errno, _("realloc failed"));
+	exit(PS_UNDEFINED);
+    }
     return p;
 }
 
